/*
 * Copyright (c) 2024 Lucas Gabriel Vuotto <lucas@lgv5.net>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stddef.h>
#include <stdint.h>


struct lc_cipher_impl {
	int	(*encrypt_init)(void *, const uint8_t *, size_t,
		    const uint8_t *, size_t);
	int	(*encrypt_update)(void *, uint8_t *, size_t *, const uint8_t *,
		    size_t);
	int	(*encrypt_final)(void *, uint8_t *, size_t *);
	int	(*encrypt)(const uint8_t *, size_t, const uint8_t *, size_t,
		    uint8_t *, size_t *, const uint8_t *, size_t);

	int	(*decrypt_init)(void *, const uint8_t *, size_t,
		    const uint8_t *, size_t);
	int	(*decrypt_update)(void *, uint8_t *, size_t *, const uint8_t *,
		    size_t);
	int	(*decrypt_final)(void *, uint8_t *, size_t *);
	int	(*decrypt)(const uint8_t *, size_t, const uint8_t *, size_t,
		    uint8_t *, size_t *, const uint8_t *, size_t);

	size_t	  argsz;
};

struct lc_cipher_ctx {
	const struct lc_cipher_impl	*impl;
	void				*arg;
};
